/*
 * SimAnnealingSkeleton.java
 *
 * Created on 16. prosinec 2005, 0:27
 *
 */

package savi.alg;

/**
 * Basic abstract class for solvers. Solver should be derived from this class 
 * if it will be used with other SAVi modules.
 * @author Premysl JIRIK
 */
public abstract class SimAnnealingSkeleton {
    /**
     * Contain reference to current state.
     */
    protected StateSkeleton   state = null;
    /**
     * Current temperature
     */
    protected double          temp;
    /**
     * Final temperature specified by parameter in constructor
     */
    protected double          finalTemp;
    /**
     * Cooling factor specified by parameter in constructor
     */
    protected double          coolFactor;
    
    
    /**
     * Creates a new instance of SimAnnealingSkeleton, initialize protected variables.
     * @param s Initial state
     * @param temp Starting temperature value
     * @param finalTemp Final temperature value.
     * @param coolFactor Cooling factor
     */
    public SimAnnealingSkeleton(StateSkeleton s, double temp, double finalTemp, double coolFactor) {
        this.finalTemp = finalTemp;
        this.coolFactor = coolFactor;
        this.temp = temp;
        
        state = s;
        state.findInitialState(temp);
    }

    /**
     * Return current state of this algorithm.
     * @return current state of this algorithm.
     */
    public StateSkeleton getState() {
        return state;
    }
    /**
     * Returns current temperature
     * @return Current temperature
     */
    public double getTemp() {
        return temp;
    }
    
    /**
     * This must be implemented, call this function lowers current temperature.
     */
    protected abstract void cool();
    /**
     * It must be implemented. Do iteration of equilibrium cycle with constant 
     * temperature.
     */
    protected abstract void equilibrium();
    /**
     * It must be implemented. This function can be called to recognition end of 
     * algorithm run.
     * @return return true if algorithm finish all work.
     */
    protected abstract boolean frozen(); 
    /**
     * Must be implemented, it should solve problem.
     * @return solution
     */
    public abstract StateSkeleton solve();
}
