/*
 * StateSAT3.java
 *
 * Created on 2. erven 2006, 23:04
 *
 */

package savi.alg.maxsat3;

import java.util.ArrayList;
import savi.alg.StateSkeleton;
import savi.alg.sat3.*;

/**
 * State representing MAX-3-SAT problem.
 * @author Premysl JIRIK
 */
public class StateMaxSAT3 implements StateSkeleton {
    /**
     * reference to formula
     */
    protected Formula           formula;
    /**
     * current variables values
     */
    protected boolean           varValues[];

    
    /**
     * Creates a new instance of StateSAT3, initialize formula
     * @param formula formula
     */
    public StateMaxSAT3(Formula formula) {
        this.formula = formula;
        varValues = new boolean[formula.varsCount()];
    }

    /**
     * Create intial solution.
     * @param temp starting temperature
     */
    public void findInitialState(double temp) {
        for (int i = 0; i < varValues.length; i++)
            varValues[i] = Math.random() < 0.5;
    }
    /**
     * Find next state.
     * @param temp current temperature
     */
    public void findNextState(double temp) {
        int idx = (int)((double)varValues.length * Math.random());
        varValues[idx] = !varValues[idx];
    }
    /**
     * Count current optimalization criterion.
     * @return value of optimalization criterion.
     */
    public double getActualCost() {
        return 1 - countSatisfactClauses() / (double)formula.getClauses().length;
    }
    
    /**
     * Return true if formula is satisfied with current variable assignment.
     * @return satisfied
     */
    public boolean isSatisfy() {
        return formula.isSatisfact(varValues);
    }

    /**
     * Count number of current satisfied clauses
     * @return number of satisfied clauses
     */
    public int countSatisfactClauses() {
        return formula.countSatisfactClauses(varValues);
    }
    
    /**
     * Creates and returns a copy of this object.
     * @return deep copy of StateMaxSAT3 object
     */
    public Object clone() {
        StateMaxSAT3 copy = null;
        try {
            copy = (StateMaxSAT3)super.clone();
            copy.varValues = this.varValues.clone();
        } 
        catch (CloneNotSupportedException ex) {
            ex.printStackTrace();
        }
        return copy;
    }
    /**
     * Returns reference to formula
     * @return reference to formula
     */
    public Formula getFormula() {
        return formula;
    }
    /**
     * Set formula to this object
     * @param f formula
     */
    public void setFormula(Formula f) {
        formula = f;
        varValues = new boolean[formula.varsCount()];
    }    
    /**
     * Returns current variable assignment
     * @return variable assignment
     */
    public boolean[] getVariableValues() {
        return varValues;
    }
    
    /**
     * Return text representation of max-3-sat instance.
     * @return array of text lines
     */
    public String [] getSubmission() {
        ArrayList al = new ArrayList();

        al.add("c saved by SAVi (Simulated Annealing VIsualization applet)");
        al.add("c Premysl Jirik, 2006, master thesis");
        al.add("c CVUT FEL (www.feld.cvut.cz)");
        al.add("c");
        al.add("p cnf " + Integer.toString(formula.varsCount()) + " " + 
                Integer.toString(formula.clausesCount()));
        
        Clause [] c = formula.getClauses();
        for (int i = 0; i < c.length; i++) {
            al.add(c[i].getCnfRepresentation());
        }
        al.add("0");
        
        String [] lines = new String[al.size()];
        for (int i = 0; i < al.size(); i++) {
            lines[i] = (String)al.get(i);
        }
        return lines;
    }
}
