/*
 * Formula.java
 *
 * Created on 2. erven 2006, 23:04
 *
 */

package savi.alg.sat3;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.Vector;
import savi.alg.weighsat3.*;


/**
 * Class representing boolean formula in CNF.
 * @author Premysl JIRIK
 */

public class Formula {
    /**
     * list of clauses in formula
     */
    protected Clause    clauses[];
    /**
     * number of variables in formula
     */
    protected int       varsCount;
    
    /**
     * Creates a new instance of Formula, randomly generate clauses from given parameters.
     * @param countClauses number of clauses in formula
     * @param varsCount number of variables in formula
     */
    public Formula(int countClauses, int varsCount) {
        this.varsCount = varsCount;
        clauses = new Clause[countClauses];
        for (int i = 0; i < countClauses; i++) {
            clauses[i] = new Clause(varsCount, 3);
        }
    }
    /**
     * Creates a new instance of Formula, read it from stream (expecting stream with
     * DIMACS format).
     * @param br stream with buffer reader
     * @throws savi.alg.sat3.WrongInputFormulaData syntax error exception
     */
    public Formula(BufferedReader br) throws WrongInputFormulaData {
        readFromStream(br);
    }

    /**
     * Return <CODE>true</CODE> if formula is satisfied with given variables assignment.
     * @param vars variables assignment
     * @return <CODE>true</CODE> if formula is satisfied
     */
    public boolean isSatisfact(boolean vars[]) {
        for (int i = 0; i < clauses.length; i++) {
            if (!clauses[i].isSatisfact(vars))
                return false;
        }
        return true;
    }
    /**
     * Return percentual satisfaction of the formula with given variables assignment.
     * 0 means no one clause is satisfied, 1 means formula is satisfied
     * @param vars variables assignment
     * @return fraction of clauses satisfaction
     */
    public double percentualSatisfact(boolean vars[]) {
        return (double)countSatisfactClauses(vars) / clauses.length;
    }
    /**
     * Return number of satisfied clauses.
     * @param vars variables assignment
     * @return number of satisfied clauses
     */
    public int countSatisfactClauses(boolean vars[]) {
        int sat = 0;
        for (int i = 0; i < clauses.length; i++) {
            if (clauses[i].isSatisfact(vars))
                sat++;                
        }
        return sat;
    }
    /**
     * Return count of variables.
     * @return count of variables
     */
    public int varsCount() {
        return this.varsCount;
    }
    /**
     *  Return count of clauses.
     * @return count of variables
     */
    public int clausesCount() {
        return this.clauses.length;
    }
    // removes clauses, that the rest size correspond to |clauses| * finalPercent
    // 0 < finalPercent <= 1
    /**
     * Reduce number of clauses in the formula.
     * @param finalPercent percentual count, how many clauses should remain
     * @throws savi.alg.sat3.WrongFormulaException the formula is reduced too many
     */
    public void reduceNumberOfClauses(double finalPercent) throws WrongFormulaException {
        int newCount = (int)((double)clausesCount() * finalPercent);

        if (newCount < 3)
            throw new WrongFormulaException();
                
        Clause [] nc = new Clause[newCount];
        int idx = 0;
        
        while (newCount > 0) {
            int rnd = (int)(Math.random() * clausesCount());
            while (clauses[rnd] == null)
                rnd = (int)(Math.random() * clausesCount());
            
            nc[idx] = clauses[rnd];
            clauses[rnd] = null;
            idx++;
            newCount--;
        }
        this.clauses = nc;
    }
    
    /**
     * Read formula from stream (expecting stream with DIMACS format).
     * @param br stream with buffer reader
     * @throws savi.alg.sat3.WrongInputFormulaData syntax error exception
     */
    protected void readFromStream(BufferedReader br) throws WrongInputFormulaData {
        try {
            String s = null;
            String vals[] = null;
            IntegerReader ir = new IntegerReader(br);

            while ((s = br.readLine()) != null) {
                if (s.length() == 0)
                    continue;
                if (s.startsWith("c"))
                    continue;
                if (s.startsWith("p cnf")) {
                    vals = s.split(" +");
                    varsCount = Integer.parseInt(vals[2]);
                    clauses = new Clause[Integer.parseInt(vals[3])];
                    break;
                }
            }
            if (s == null)
                throw new WrongInputFormulaData();

            Vector dimacs = new Vector();
            for (int i = 0; i < clauses.length; i++) {
                dimacs.clear();
                int x = ir.readInt();
                while (x != 0) {
                    dimacs.add(x);
                    x = ir.readInt();
                }
                clauses[i] = new Clause(dimacs);
            }
        } catch (IOException ie) {
            throw new WrongInputFormulaData();
        }
    }
    
    /**
     * Return number of clauses.
     * @return number of clauses
     */
    public Clause[] getClauses() {
        return clauses;
    }
    
    /**
     * Creates and returns a copy of this object.
     * @return deep copy of Formula object
     */
    public Formula clone() {
        Formula f = new Formula(1, 1);
        f.varsCount = this.varsCount;
        f.clauses = this.clauses.clone();
        return f;
    }
}
