/*
 * StateTSP.java
 *
 * Created on 9. nor 2006, 13:34
 *
 */

package savi.alg.tsp;

import java.text.*;
import savi.alg.*;
import java.util.*;
/**
 * State representing TSP problem
 * @author Premysl JIRIK
 */
public class StateTSP implements StateSkeleton {
    /**
     * reference to list of cities
     */
    protected Cities cities;
    /**
     * current cost (length of tour)
     */
    protected double cost = 0;
    /**
     * current path
     */
    protected int path[];
    /**
     * instance to random generator
     */
    protected Random rnd;
    
    /**
     * Creates a new instance of StateTSP, initialize cities
     * @param cities cities list
     */
    public StateTSP(Cities cities) {
        this.cities = cities;
        path = new int[cities.count];
        rnd = new Random();
    }
    /**
     * Reverse path between two cities in given city list
     * @param array city list
     * @param start starting index
     * @param end final index
     */
    protected void reversePath(int array[], int start, int end) {
        int length = end - start;
        if(length < 0)
            length += array.length;
        length = length / 2 + 1;
        for(int i = 0; i < length; i++)
        {
            int temp = array[start];
            array[start] = array[end];
            array[end] = temp;
            if(++start >= array.length)
                start = 0;
            if(--end < 0)
                end = array.length-1;
        }
    }
    /**
     * Find initial state.
     * @param temp starting temp
     */
    public void findInitialState(double temp) {
        cost = 0;
        for (int i = 0; i < path.length; i++) {
            path[i] = i;
            cost += cities.distance(i, i+1);
        }
    }
    /**
     * Find next state.
     * @param temp current temperature
     */
    public void findNextState(double temp) {
        int a = rnd.nextInt(path.length);   // % path.length;
        int b = rnd.nextInt(path.length);   // % path.length;
        int anext = (a + path.length - 1) % path.length;
        int bnext = (b + 1) % path.length;
        double deltaCostOld = cities.distance(path[a], path[anext]) 
                            + cities.distance(path[b], path[bnext]);
        //swap
        reversePath(path, a, b);
        //cost
        cost += -deltaCostOld + cities.distance(path[a], path[anext]) + cities.distance(path[b], path[bnext]);
    }
    /**
     * Return current optimalization criterion value.
     * @return opt. criterion value
     */
    public double getActualCost() {
        return cost;
    }
    /**
     * Create a copy of this object.
     * @return a copy of this object.
     */
    public Object clone() {
        StateTSP copy = null;
        try {
            copy = (StateTSP)super.clone();
            copy.path = this.path.clone();
        }
        catch (CloneNotSupportedException e) {
            e.printStackTrace();
        }
        return copy;
    }
    /**
     * Recalculate path length.
     * @return path length
     */
    public double recountCost() {
        double c = 0;
        for (int i = 0; i < (path.length - 1); i++) 
            c += cities.distance(path[i], path[i+1]);
        c += cities.distance(path[path.length - 1], path[0]);
        return c;
    }

    /**
     * Return path sequence
     * @return path sequence
     */
    public int[] getPath() {
        return path;
    }
    /**
     * Return list of cities
     * @return a list of cities
     */
    public Cities getCities() {
        return cities;
    }
    /**
     * Set list of cities
     * @param c cities list
     */
    public void setCities(Cities c) {
        this.cities = c;
        path = new int[cities.count];
        findInitialState(0);       // temp, it does not matter
    }

    /**
     *  Return text representation of TSP instance.
     * @return array of lines
     */
    public String [] getSubmission() {
        ArrayList al = new ArrayList();
        DecimalFormat df = new DecimalFormat("##0.0000", new DecimalFormatSymbols(Locale.ENGLISH));
        
        al.add("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        al.add("<tsp>");
        
        for (int i = 0; i < cities.count; i++) {
            al.add(" <city>");
            al.add("  <x>" + df.format(cities.getCityX(i)) + "</x>");
            al.add("  <y>" + df.format(cities.getCityY(i)) + "</y>");
            al.add(" </city>");
        }
        al.add("</tsp>");
        
        String [] lines = new String[al.size()];
        for (int i = 0; i < al.size(); i++) {
            lines[i] = (String)al.get(i);
        }
        return lines;
    }

}
