/*
 * WeightedSAT3.java
 *
 * Created on 4. z 2006, 9:40
 *
 */

package savi.visualization.addons;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.*;
import java.awt.*;
import java.net.*;
import java.text.DecimalFormat;
import java.io.*;
import savi.alg.StateSkeleton;

import savi.dialogs.*;
import savi.alg.sat3.*;
import savi.alg.weighsat3.*;
import savi.visualization.support.VisualizationInfo;
import savi.visualization.support.SimPanel;

/**
 * Visualization add-on for MAX-WEIGHTED-3-SAT problem.
 * @author Premysl JIRIK
 */
public class WeightedSAT3 extends SimPanel {
    WeightedFormula         formula;
    Label                   lbSatisfy, lbCntSatisfact, lbCntClauses, lbWeight;

    /** Creates a new instance of Knapsack */
    public WeightedSAT3() {
        super(new ViewWeightedSAT3(), 2, 0.0001, 0.99, 200);
    }
    
    /**
     * Initialize problem state.
     */
    protected void initState() {
        state = new StateWSAT3(formula);
    }
    
    /**
     * Prepare and return help panel for this problem.
     * @return help panel
     */
    protected VisualizationInfo getInfoPanel() {
        VisualizationInfo vi = new VisualizationInfo();
        vi.addTab("Description", "/res/help/wsat/wsat01.gif");
        vi.addTab("Controls", "/res/help/wsat/wsat02.gif");
        vi.addTab("Visualization", "/res/help/wsat/wsat03.gif");
        vi.addTab("WCNF file format", "/res/help/wsat/wsat04.gif");
        return vi;
    }

    /**
     * Return name of the problem.
     * @return name of the problem
     */
    public String getSimulationName() {
        return "Maximum weighted 3-satisfiability (WSAT3)";
    }    

    /**
     * Initialize components in the add-on.
     */
    protected void initComponents() {
        super.initComponents();
        
        addInputButton("Reduce formula").addActionListener(new BrReduceFormula());
        addInputButton("Random form.").addActionListener(new BtRandomItems());
        addInputButton("Save instance").addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                callSaveInstance();
            }
        });
        addInputButton("Load from URL").addActionListener(new BtUrlLoad());
        addInputButton("Load from file").addActionListener(new BtFileChoose());
        
        lbSatisfy = addOutputField("Satisfied", "");
        lbCntClauses = addOutputField("Clauses count", "");
        lbCntSatisfact = addOutputField("Satisfied cl.", "");
        lbWeight = addOutputField("Current weight", "");
        
        formula = new WeightedFormula(91, 20);
        
        settings.doLayout();
    }   
    
    /**
     * Update info panel (display) with given state.
     * @param s state
     */
    protected void updateDisplay(StateSkeleton s) {
        super.updateDisplay(s);
        StateWSAT3 st = (StateWSAT3)s;
        if (st.isSatisfy())
            lbSatisfy.setText("YES");
        else
            lbSatisfy.setText("NO");
        
        lbCntSatisfact.setText(Integer.toString(st.countSatisfactClauses()));
        lbCntClauses.setText(Integer.toString(st.getFormula().clausesCount()));
        lbWeight.setText(Integer.toString((int)st.getWeight()));
    }

    /**
     * Load instance of the problem from stream with bufferedreader.
     * @param br stream with bufferedreader
     */
    protected void loadFromStream(BufferedReader br) {
        try {
            WeightedFormula wf;
            wf = new WeightedFormula(br);
            formula = wf;
            state = new StateWSAT3(wf);
            initAlg();
            updateDisplay();
        } catch (WrongInputFormulaData ex) {
            new SaviErrorMessage("WrongInputFormulaData: submission contains syntax error.");
        } catch (Exception ex) {
            new SaviErrorMessage(ex.toString());
        }
    }

    class BtFileChoose implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            running(false);
            FileDialog fd = new FileDialog(new Frame(""), "WeightedSAT submission", FileDialog.LOAD);
            fd.setVisible(true);

            String curFile;
            if ((curFile = fd.getFile()) != null) {
                String filename = fd.getDirectory() + curFile;
                File f = new File(filename);
                
                try {
                    loadFromStream(new BufferedReader(
                            new InputStreamReader( 
                                new FileInputStream(f))));
                } catch (FileNotFoundException ex) {
                    new SaviErrorMessage("File Not Found: " + filename);
                }
            }
        }
    }
    
    class BtUrlLoad implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            running(false);
            SaviUrlDialog sud = new SaviUrlDialog("WeightedSAT - load submission from URL");
            if (sud.showForm()) {
                try {
                    URL url = new URL(sud.getURL());
                    URLConnection urlc = (URLConnection)url.openConnection();
                    String userAgent = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8) Gecko/20051111 Firefox/1.5";
                    urlc.setRequestProperty("User-Agent", userAgent );      // Identify as Firefox 1.5 ;)
                    urlc.setRequestProperty( "Connection", "close" );       // optionally turn off keep-alive
                    urlc.setAllowUserInteraction(false);
                    urlc.setDoInput(true);
                    urlc.setDoOutput(false);
                    urlc.setUseCaches(false);
                    urlc.connect();

                    long length = urlc.getContentLength();      // -1 if not available
                    InputStream is = urlc.getInputStream();

                    loadFromStream(new BufferedReader(new InputStreamReader(is)));
                    is.close();
                } catch (MalformedURLException ex) {
                    new SaviErrorMessage("Malformed URL, string could not be parsed.");
                } catch (IOException ex) { 
                    new SaviErrorMessage("Cannot open input file.");
                } catch (Exception ex) { 
                    new SaviErrorMessage(ex.toString());
                }
            }
        }
    }
            
    class BtRandomItems implements ActionListener {
        int init_vars = 21, init_claus = 90;
        
        public void actionPerformed(ActionEvent e) {
            running(false);
            SaviDialog sd = new SaviDialog("WeightedSAT generator");
            TextField fieldVars, fieldClauses;
            sd.addInputRow("Variables count", fieldVars = new TextField("20", 3));
            sd.addInputRow("Clauses count", fieldClauses = new TextField("91", 4));
            fieldVars.setText(Integer.toString(init_vars));
            fieldClauses.setText(Integer.toString(init_claus));
            if (sd.showForm()) {
                int vars, claus;
                try {
                    vars = (int)Double.parseDouble(fieldVars.getText());
                    if ((vars < 4) || (vars > 500))
                        throw new Exception();
                } catch (Exception ex) {
                    new SaviErrorMessage("Variables count must be a numeric type between 5 and 500.");
                    return;
                }
                try {
                    claus = (int)Double.parseDouble(fieldClauses.getText());
                    if ((claus < 3) || (claus > 2500))
                        throw new Exception();
                } catch (Exception ex) {
                    new SaviErrorMessage("Clauses count must be a numeric type between 5 and 2500.");
                    return;
                }
                ((StateWSAT3)state).setFormula(new WeightedFormula(claus, vars));
                initAlg();
                updateDisplay();
                init_vars = vars;
                init_claus = claus;
            }
        }
    }
    
    class BrReduceFormula implements ActionListener {
        double init_percentage = 80;
        
        public void actionPerformed(ActionEvent e) {
            running(false);
            SaviDialog sd = new SaviDialog("WeightedSAT - formula reducing");
            TextField fieldRed;
            sd.addInputRow("Reduce to [%]", fieldRed = new TextField("80", 2));
            fieldRed.setText(Double.toString(init_percentage));
            if (sd.showForm()) {
                double percentage;
                try {
                    percentage = Double.parseDouble(fieldRed.getText());
                    if ((percentage <= 0) || (percentage >= 100))
                        throw new NumberFormatException();

                    WeightedFormula wf = formula.clone();
                    wf.reduceNumberOfClauses(percentage / 100);
                    ((StateWSAT3)state).setFormula(wf);
                    formula = wf;
                    initAlg();
                    updateDisplay();
                    init_percentage = percentage;
                } catch (NumberFormatException ex) {
                    new SaviErrorMessage("Value must be a number between 1 and 100.");
                } catch (WrongFormulaException ex) {
                    new SaviErrorMessage("Formula is too simple.");
                } catch (Exception ex) {
                    new SaviErrorMessage(ex.toString());
                }
            }
            
        }
    }
    
}
