/*
 * VisualizationGraph.java
 *
 * Created on 26. srpen 2006, 15:11
 *
 */

package savi.visualization.support;

import java.awt.*;
import java.text.DecimalFormat;
/**
 * Provides graph visualization.
 * @author Premysl JIRIK
 */
public class VisualizationGraph extends Panel {
    /**
     * list of y-values
     */
    protected double [] optCritValues;
    /**
     * number of y-values
     */
    protected int       valCount;

    Font                fontText, fontCaption, fontValues;
    Image               buffImage;
    Graphics            offscreen;
    int                 imgWidth, imgHeight;
    boolean             initDone = false;

    
    /** Creates a new instance of VisualizationGraph */
    public VisualizationGraph() {
        imgWidth = 420;
        imgHeight = 445;
        this.setBackground(Color.WHITE);
        fontCaption = new Font("SansSerif", Font.BOLD, 18);
        fontText = new Font("Serif", 0, 15);
        fontValues = new Font("Serif", 0, 12);
    }
    
    /**
     * Initialize internal structures for given SA parameters.
     * @param stemp starting temperature
     * @param ftemp final temperature
     * @param coolf cooling factor
     */
    public void initialize(double stemp, double ftemp, double coolf) {
        double t = stemp;
        int cnt = 0;
        while (t > ftemp) {
            cnt++;
            t *= coolf;
        }
        optCritValues = new double[cnt];
        valCount = 0;
    }
    /**
     * Cause update content.
     */
    public void updateScene() {
        refresh();
    }    
    
    /**
     * Adds new value into graph.
     * @param v y-value
     */
    public void addValue(double v) {
        if (valCount >= optCritValues.length)
            return;
        optCritValues[valCount++] = v;
    }
    
    /**
     * Paints this component.
     * @param g the graphics context to use for painting
     */
    public void paint(Graphics g) {
        if (!initDone)
            initpaint(g);
        g.drawImage(buffImage, 0, 0, this);
    }
    /**
     * Updates this component.
     * @param g the specified context to use for updating
     */
    public void update(Graphics g) {
        if (!initDone)
            initpaint(g);
        g.drawImage(buffImage, 0, 0, this);
    }
    /**
     * Cause refresh (graph repaint).
     */
    public void refresh() {
        initDone = false;
        repaint();
    }
    /**
     * Initialize graph picture or repaint it.
     * @param g the specified context to use for painting
     */
    protected void initpaint(Graphics g) {
        if (offscreen == null) {
            buffImage = this.createImage(imgWidth, imgHeight);
            offscreen = buffImage.getGraphics();
        }
        
        offscreen.setColor(Color.WHITE);
        offscreen.fillRect(0, 0, imgWidth, imgHeight);
        
        offscreen.setColor(Color.BLACK);

        offscreen.setFont(fontCaption);
        String s = "Graph of optimalization criterion";
        int swidth = offscreen.getFontMetrics().stringWidth(s);
        offscreen.drawString(s, (imgWidth - swidth) / 2, 25);
        
        offscreen.drawLine(40, imgHeight - 30, imgWidth - 10, imgHeight - 30);      // x
        offscreen.drawLine(40, 50, 40, imgHeight - 30);                             // y
        
        offscreen.setFont(fontText);
        s = "steps";
        swidth = offscreen.getFontMetrics().stringWidth(s);
        offscreen.drawString(s, imgWidth - 20 - swidth, imgHeight - 15);

        s = "opt. criterion";
        swidth = offscreen.getFontMetrics().stringWidth(s);
        offscreen.drawString(s, 3, 45);
        
        
        int xPoints[] = {imgWidth - 10, imgWidth - 17, imgWidth - 17, imgWidth - 10};
        int yPoints[] = {imgHeight - 30, imgHeight - 26, imgHeight - 34, imgHeight - 30};
        offscreen.fillPolygon(xPoints, yPoints,  4);
        int xP2[] = { 40, 44, 36, 40 };
        int yP2[] = { 50, 57, 57, 50 };
        offscreen.fillPolygon(xP2, yP2, 4);
        
        offscreen.setFont(fontValues);
        for (int i = 0; i < 4; i++) {
            int x = 40 + (int)((double)i / 4.0 * (double)(imgWidth - 65));
            int val  = (int)((double)i / 4.0 * (double)optCritValues.length);
            String sval = Integer.toString(val);
            int slen = offscreen.getFontMetrics().stringWidth(sval);
            offscreen.drawLine(x, imgHeight - 32, x, imgHeight - 28);
            offscreen.drawString(sval, x - slen / 2, imgHeight - 15);
        }
        int heightMax = imgHeight - 30 - 65;
        double max = getMaxValue();
        if (max == 0)
            max = 1;
        DecimalFormat df = new DecimalFormat("0.#E0");
        for (int i = 0; i < 5; i++) {
            int y = (imgHeight - 30) - (int)((double)heightMax * (double)i / 4.0);
            double val  = (double)i / 4.0 * (double)max;
            String sval = df.format(val);
            if (offscreen.getFontMetrics().stringWidth(sval) > 
                    offscreen.getFontMetrics().stringWidth(Double.toString(val))) {
                sval = Double.toString(val);
            }
            int slen = offscreen.getFontMetrics().stringWidth(sval) + 3;
            offscreen.drawLine(38, y, 42, y);
            int x = 40 - slen;
            if (x < 0)
                x = 0;
            offscreen.drawString(sval, x, y + 5);
        }
        
        offscreen.setColor(Color.BLACK);
        max = getMaxValue();
        int maxVals = optCritValues.length;
        if (maxVals == 0)
            return;
        for (int i = 0; i < valCount; i++) {
            int x = 41 + (int)((double)(imgWidth - 65) * (double)i / (double)maxVals);
            int y = (imgHeight - 30) - (int)((imgHeight - 30 - 65) * optCritValues[i] / max);
            offscreen.drawRect(x, y, 1, 1);
        }
    }
    
    /**
     * Return maximum y-value.
     * @return y-value
     */
    protected double getMaxValue() {
        double max = 0;
        for (int i = 0; i < valCount; i++) 
            if (max < optCritValues[i])
                max = optCritValues[i];
        return max;
    }
}
